from giveagame.sprites.Sprite import *
from giveagame.Coords import *

class PlatformSprite(Sprite):
    def __init__(self, game, photo_image, x, y, width, height):
        Sprite.__init__(self, game)
        self.photo_image = photo_image
        self.image = game.canvas.create_image(x, y, \
                                              image=self.photo_image, anchor='nw')
        self.coordinates = Coords(x, y, x + width, y + height)


from tkinter import *
from giveagame.Coords import *
from giveagame.sprites.DoorSprite import *
from giveagame.sprites.MovingPlatformSprite import *
from giveagame.sprites.PlatformSprite import *
from giveagame.sprites.Sprite import *
from giveagame.sprites.StickFigureSprite import * 

class Game:
	def __init__(self, title, cw, ch, bg_img):
		self.tk = Tk()
		self.tk.title(title)
		self.tk.resizable(0, 0)
		self.tk.wm_attributes("-topmost", 1)
		self.canvas = Canvas(self.tk, width=cw, height=ch, \
			highlightthickness=0)
		self.canvas.pack()
		self.tk.update()
		self.canvas_height = ch
		self.canvas_width = cw
		self.bg = PhotoImage(file=bg_img)
		self.icon = PhotoImage(file="imgs/icon.gif")
		self.tk.iconphoto(False, self.icon)
		w = self.bg.width()
		h = self.bg.height()
		for x in range(0, 5):
			for y in range(0, 5):
				self.canvas.create_image(x * w, y * h, \
					image=self.bg, anchor='nw')
		self.sprites = []
		self.running = True

	def mainloop(self):
		while 1:
			if self.running == True:
				for sprite in self.sprites:
					sprite.move()
			self.tk.update_idletasks()
			self.tk.update()
			time.sleep(0.01)



from tkinter import *
from tkinter import messagebox
import webbrowser

class GUI:
	def __init__(self, root, game):
		self.tk = root
		self.mainMenu = Menu(self.tk)
		self.index = 0
		self.game = game
		self.tk.config(menu=self.mainMenu)
		self.settingsMenu = Menu(self.mainMenu, tearoff=0)
		self.settingsMenu.add_command(label="Pause.", command=self.com_PauseGame, accelerator="Ctrl+P")
		self.settingsMenu.add_separator()
		self.settingsMenu.add_command(label="Exit.", command=self.com_Exit, accelerator="Ctrl+Q")
		self.helpMenu = Menu(self.mainMenu, tearoff=0)
		self.helpMenu.add_command(label="README file.", command=self.com_OpenReadmeFile, accelerator="Ctrl+H")
		self.helpMenu.add_separator()
		self.helpMenu.add_command(label="About programm.", command=self.com_OpenAboutProgram)
		self.mainMenu.add_cascade(label="Settings.", menu=self.settingsMenu)
		self.mainMenu.add_cascade(label="Help.", menu=self.helpMenu)

		self.tk.bind_all("<Control-q>", self.com_Exit)
		self.tk.bind_all("<Control-p>", self.com_PauseGame)
		self.tk.bind_all("<Control-h>", self.com_OpenReadmeFile)

	def com_Exit(self):
		self.tk.destroy()
		quit()

	def com_OpenReadmeFile(self):
		webbrowser.open_new_tab('README.html')

	def com_OpenAboutProgram(self):
		messagebox.showinfo(title="About programm.", message="Make on GiveAGame. 0.0.3")

	def com_PauseGame(self):
		self.index = not self.index

		if self.index:
			self.game.running = False
		else:
			self.game.running = True



from giveagame.sprites.Sprite import *
from giveagame.sprites.PlatformSprite import *
import time

class MovingPlatformSprite(PlatformSprite):
    def __init__(self, game, photo_image, x, y, width, height):
        PlatformSprite.__init__(self, game, photo_image, x, y, \
                                width, height)
        self.x = 2
        self.counter = 0
        self.last_time = time.time()
        self.width = width
        self.height = height

    def coords(self):
        xy = self.game.canvas.coords(self.image)
        self.coordinates.x1 = xy[0]
        self.coordinates.y1 = xy[1]
        self.coordinates.x2 = xy[0] + self.width
        self.coordinates.y2 = xy[1] + self.height
        return self.coordinates

    def move(self):
        if time.time() - self.last_time > 0.03:
                self.last_time = time.time()
                self.game.canvas.move(self.image, self.x, 0)
                self.counter = self.counter + 1
                if self.counter > 20:
                    self.x = self.x * -1
                    self.counter = 0


from threading import Thread
from playsound import playsound

class Music:
    def __init__(self, music_file, loop_music):
        self.music_file = music_file
        self.loop_music = loop_music

    def music(self):
        if self.loop_music == True:
            while self.loop_music:
                playsound(self.music_file, block=False)
        else:
            playsound(self.music_file, block=True)

    def start(self):
        self.Thread(target=music, daemon=True, name="musicClassThread").start()



from giveagame.sprites.Sprite import *
from giveagame.Coords import *
from giveagame.Music import *
from tkinter import PhotoImage
import time

class StickFigureSprite(Sprite):
    def __init__(self, game):
        Sprite.__init__(self, game)
        self.images_left = [
            PhotoImage(file="imgs/figure-L1.gif"),
            PhotoImage(file="imgs/figure-L2.gif"),
            PhotoImage(file="imgs/figure-L3.gif")
        ]
        self.images_right = [
            PhotoImage(file="imgs/figure-R1.gif"),
            PhotoImage(file="imgs/figure-R2.gif"),
            PhotoImage(file="imgs/figure-R3.gif")
        ]
        self.sound_jump_file = "sounds/jmp.waw"
        self.sound_jump = new giveagame.Music(sound_jump_file, False)
        self.image = game.canvas.create_image(200, 470, \
                                              image=self.images_left[0], anchor='nw')
        self.x = -2
        self.y = 0
        self.current_image = 0
        self.current_image_add = 1
        self.jump_count = 0
        self.last_time = time.time()
        self.coordinates = Coords()
        game.canvas.bind_all('<KeyPress-Left>', self.turn_left)
        game.canvas.bind_all('<KeyPress-Right>', self.turn_right)
        game.canvas.bind_all('<space>', self.jump)

    def turn_left(self, evt):
        if self.y == 0:
            self.x = -2

    def turn_right(self, evt):
        if self.y == 0:
            self.x = 2

    def jump(self, evt):
        if self.y == 0:
            self.sound_jump.start()
            self.y = -4
            self.jump_count = 0

    def animate(self):
        if self.x != 0 and self.y == 0:
            if time.time() - self.last_time > 0.1:
                self.last_time = time.time()
                self.current_image += self.current_image_add
                if self.current_image >= 2:
                    self.current_image_add = -1
                if self.current_image <= 0:
                    self.current_image_add = 1
                    
        if self.x < 0:
            if self.y != 0:
                self.game.canvas.itemconfig(self.image, \
                                            image=self.images_left[2])
            else:
                self.game.canvas.itemconfig(self.image, \
                                            image=self.images_left[ \
                                                self.current_image])

        elif self.x > 0:
            if self.y != 0:
                self.game.canvas.itemconfig(self.image, \
                                            image=self.images_right[2])
            else:
                self.game.canvas.itemconfig(self.image, \
                                            image=self.images_right[ \
                                                self.current_image])

    def coords(self):
        xy = self.game.canvas.coords(self.image)
        self.coordinates.x1 = xy[0]
        self.coordinates.y1 = xy[1]
        self.coordinates.x2 = xy[0] + 27
        self.coordinates.y2 = xy[1] + 30
        return self.coordinates

    def move(self):
        self.animate()
        if self.y < 0:
            self.jump_count += 1
            if self.jump_count > 20:
                self.y = 4
        if self.y > 0:
            self.jump_count -= 1
        co = self.coords()
        left = True
        right = True
        top = True
        bottom = True
        falling = True
        if self.y > 0 and co.y2 >= self.game.canvas_height:
            self.y = 0
            bottom = False
        elif self.y < 0 and co.y1 <= 0:
            self.y = 0
            top = False
        if self.x > 0 and co.x2 >= self.game.canvas_height:
            self.x = 0
            right = False
        elif self.x < 0 and co.x1 <= 0:
            self.x = 0
            left = False
        for sprite in self.game.sprites:
            if sprite == self:
                continue
            sprite_co = sprite.coords()
            if top and self.y < 0 and collided_top(co, sprite_co):
                self.y = -self.y
                top = False
            if bottom and self.y > 0 and collided_bottom(self.y, \
                                                         co, sprite_co):
                self.y = sprite_co.y1 - co.y2
                if self.y < 0:
                    self.y = 0
                bottom = False
                top = False
            if bottom and falling and self.y == 0 \
               and co.y2 < self.game.canvas_height \
               and collided_bottom(1, co, sprite_co):
                falling = False
            if left and self.x < 0 and collided_left(co, \
                                                     sprite_co):
                self.x = 0
                left = False
                if sprite.endgame:
                    self.end(sprite)
            if right and self.x > 0 and \
               collided_right(co, sprite_co):
                self.x = 0
                right = False
                if sprite.endgame:
                    self.end(sprite)
        if falling and bottom and self.y == 0 \
               and co.y2 < self.game.canvas_height:
                self.y = 4
        self.game.canvas.move(self.image, self.x, self.y)

    def end(self, sprite):
        self.game.running = False
        sprite.opendoor()
        time.sleep(1)
        self.game.canvas.itemconfig(self.image, state='hidden')
        sprite.closedoor()
        self.game.canvas.create_text(250, 250, text='Победа!', \
            font=('Helvetica', 20))
        if self.endgame and self.game.levelIndex == False:
            self.game.levelIndex = True
        if self.endgame and self.game.levelIndex == True:
            self.game.tk.update()
            time.sleep(1)
            self.game.tk.destroy()
            quit()



class Sprite:
    def __init__(self, game):
        self.game = game
        self.endgame = False
        self.coordinates = None
    def move(self):
        pass
    def coords(self):
        return self.coordinates


class Coords:
    def __init__(self, x1=0, y1=0, x2=0, y2=0):
        self.x1 = x1
        self.y1 = y1
        self.x2 = x2
        self.y2 = y2

def within_x(co1, co2):
    if (co1.x1 > co2.x1 and co1.x1 < co2.x2) \
      or (co1.x2 > co2.x1 and co1.x2 < co2.x2) \
      or (co2.x1 > co1.x1 and co2.x1 < co1.x2) \
      or (co2.x2 > co1.x1 and co2.x2 < co1.x2):
         return True
    else:
        return False
    
def within_y(co1, co2):
     if (co1.y1 > co2.y1 and co1.y1 < co2.y2) \
        or (co1.y2 > co2.y1 and co1.y2 < co2.y2) \
        or (co2.y1 > co1.y1 and co2.y1 < co1.y2) \
        or (co2.y2 > co1.y1 and co2.y2 < co1.y2):
         return True
     else:
         return False

def collided_left(co1, co2):
    if within_y(co1, co2):
       if co1.x1 <= co2.x2 and co1.x1 >= co2.x1:
           return True
    return False

def collided_right(co1, co2):
    if within_y(co1, co2):
        if co1.x2 >= co2.x1 and co1.x2 <= co2.x2:
            return True
    return False

def collided_top(co1, co2):
    if within_x(co1, co2):
        if co1.y1 <= co2.y2 and co1.y1 >= co2.y1:
            return True
    return False

def collided_bottom(y, co1, co2):
    if within_x(co1, co2):
        y_calc = co1.y2 + y
        if y_calc >= co2.y1 and y_calc <= co2.y2:
            return True
    return False


from giveagame.sprites.Sprite import *
from giveagame.Coords import *
from tkinter import PhotoImage

class DoorSprite(Sprite):
    def __init__(self, game,  x, y, width, height):
        Sprite.__init__(self, game)
        self.closed_door = PhotoImage(file="imgs/door1.gif")
        self.open_door = PhotoImage(file="imgs/door2.gif")
        self.image = game.canvas.create_image(x, y, \
                                              image=self.closed_door, anchor='nw')
        self.coordinates = Coords(x, y, x + (width / 2), y + height)
        self.endgame = True

    def opendoor(self):
        self.game.canvas.itemconfig(self.image, image=self.open_door)
        self.game.tk.update_idletasks()
        
    def closedoor(self):
        self.game.canvas.itemconfig(self.image, image=self.closed_door)
        self.game.tk.update_idletasks()
